@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2016-2021 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2016-2022 <tsujan2000@gmail.com>
  *
  * FeatherNotes is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,6 +19,7 @@
 #define FN_H
 
 //#include <QtGui> // too much
+#include <QPointer>
 #include <QListWidgetItem>
 #include <QSystemTrayIcon>
 #include <QMainWindow>
@@ -42,10 +43,17 @@ public:
     explicit FN (const QStringList& message, QWidget *parent = nullptr);
     ~FN();
 
+    void activateFNWindow (bool noDelay = false);
+    void quitting();
+
     void writeGeometryConfig (bool withLastNodeInfo = true);
     void rememberLastOpenedFile (bool recentNumIsSet = false);
     void writeConfig();
 
+    QString currentPath() const {
+        return xmlPath_;
+    }
+
     bool isSizeRem() const {
         return remSize_;
     }
@@ -88,20 +96,6 @@ public:
     void remPosition (bool rem) {
         remPosition_ = rem;
     }
-    void setPosition (const QPoint &pos) {
-        position_ = pos;
-    }
-
-    bool isUnderE() const {
-        return underE_;
-    }
-    void setUnderE (bool yes);
-    QSize EShift() const {
-        return EShift_;
-    }
-    void setEShift (QSize shift) {
-        EShift_ = shift;
-    }
 
     bool hasTray() const {
         return hasTray_;
@@ -217,6 +211,21 @@ public:
         openReccentSeparately_ = separately;
     }
 
+    void setCollapsedStates();
+    bool getRememberExpanded() const {
+        return rememberExpanded_;
+    }
+    void setRememberExpanded (bool remember) {
+        rememberExpanded_ = remember;
+    }
+
+    bool getSaveOnExit() const {
+        return saveOnExit_;
+    }
+    void setSaveOnExit (bool saveOnExit) {
+        saveOnExit_ = saveOnExit;
+    }
+
 #ifdef HAS_HUNSPELL
     QString getDictPath() const {
         return dictPath_;
@@ -226,13 +235,12 @@ public:
     }
 #endif
 
-public slots:
-    void quitting();
-
 private slots:
     bool close();
     void checkTray();
     void showContextMenu (const QPoint &p);
+    void indexExpanded (const QModelIndex &index);
+    void indexCollapsed (const QModelIndex &index);
     void fullScreening();
     void rehighlight (TextEdit *textEdit);
     void zoomingIn();
@@ -255,7 +263,7 @@ private slots:
     void setCursorInsideSelection (bool sel);
     void txtContextMenu (const QPoint &p);
     void copyLink();
-    void selChanged (const QItemSelection &selected, const QItemSelection&);
+    void selChanged (const QItemSelection &selected, const QItemSelection &deselected);
     void setSaveEnabled (bool modified);
     void setUndoEnabled (bool enabled);
     void setRedoEnabled (bool enabled);
@@ -306,9 +314,8 @@ private slots:
     void toggleWrapping();
     void toggleIndent();
     void prefDialog();
-    void noteModified();
-    void docProp();
     void nodeChanged (const QModelIndex&, const QModelIndex&);
+    void treeChanged();
     void showHideSearch();
     void clearTagsList (int);
     void selectRow (QListWidgetItem *item);
@@ -340,8 +347,6 @@ private slots:
     void updateRecenMenu();
     void openRecentFile();
     void clearRecentMenu();
-    void closeTagsDialog();
-    void updateNodeActions();
 
 #ifdef HAS_HUNSPELL
     void checkSpelling();
@@ -354,9 +359,11 @@ private:
     void createTrayIcon();
     void closeEvent (QCloseEvent *event);
     void resizeEvent (QResizeEvent *event);
+    void changeEvent (QEvent *event);
     void showEvent (QShowEvent *event);
     void showDoc (QDomDocument &doc, int node = -1);
     void setTitle (const QString &fname);
+    void showWarningBar (const QString &message, int timeout);
     void notSavedOrOpened (bool notSaved);
     void setNodesTexts();
     bool unSaved (bool modified);
@@ -372,21 +379,27 @@ private:
     void findInNames();
     bool isImageSelected();
     void readShortcuts();
-    QString validatedShortcut (const QVariant v, bool *isValid);
+    QString validatedShortcut (const QVariant v, QStringList &added, bool *isValid);
     void readAndApplyConfig (bool startup = true);
     QString nodeAddress (QModelIndex index);
-    bool isPswrdCorrect();
+    bool isPswrdCorrect (const QString &file);
     void dragMoveEvent (QDragMoveEvent *event);
     void dragEnterEvent (QDragEnterEvent *event);
     void dropEvent (QDropEvent *event);
     bool event (QEvent *event);
+    void noteModified();
+    void docProp();
+    bool hasBlockingDialog();
+    void closeWarningBar();
+    void closeNonModalDialogs();
+    void closeWinDialogs();
+    void updateNodeActions();
 
 #ifdef HAS_HUNSPELL
     void spellingCheckingMsg (const QString &msg, bool hasInfo);
 #endif
 
     Ui::FN *ui;
-    bool isX11_, isWayland_;
     bool closed_; // Whether FN::closeEvent() is called before quitting.
     //QWidget *dummyWidget; // For hiding the main window while keeping all its state info.
     QSystemTrayIcon *tray_;
@@ -398,24 +411,23 @@ private:
     QColor lastTxtColor_, lastBgColor_;
     DomModel *model_;
     QString xmlPath_;
-    LineEdit *ImagePathEntry_, *htmlPahEntry_;
+    QPointer<LineEdit> imagePathEntry_, htmlPahEntry_;
     /* By pairing each widget with a DOM item, we won't need to worry
        about keeping the correspondence between widgets and nodes: */
-    QHash<DomItem*,TextEdit*> widgets_;
+    QHash<DomItem*, TextEdit*> widgets_;
     QTextDocument::FindFlags searchFlags_; // Whole word and case sensitivity flags.
-    QHash<TextEdit*,QString> searchEntries_; // Search entries, one for each QTextEdit.
+    QHash<TextEdit*, QString> searchEntries_; // Search entries, one for each QTextEdit.
     bool searchingOtherNode_; // Needed when jumping to another node during search.
     bool rplOtherNode_; // Like above but for replacement.
     int replCount_; // Needed for counting replacements in all nodes.
-    QHash<TextEdit*, QList<QTextEdit::ExtraSelection> > greenSels_; // For replaced matches.
+    QHash<TextEdit*, QList<QTextEdit::ExtraSelection>> greenSels_; // For replaced matches.
     QString txtReplace_; // The replacing text.
     QModelIndexList tagsList_;
     QString linkAtPos_; // Text hyperlink at the right-click position.
     int imgScale_; QString lastImgPath_;
     int recentNum_;
-    bool openReccentSeparately_;
-    bool shownBefore_,
-         remSize_, remSplitter_,
+    bool remSize_, isMaxed_, isFull_,
+         remSplitter_,
          remPosition_,
          hasTray_,
          minToTray_,
@@ -428,16 +440,18 @@ private:
          autoBracket_,
          autoReplace_,
          openLastFile_,
-         treeViewDND_;
+         openReccentSeparately_,
+         rememberExpanded_,
+         saveOnExit_;
+    bool shownBefore_, treeViewDND_; // Used internally.
     int lastNode_;
     QString dateFormat_;
     int autoSave_;
     QPoint position_; // Excluding the window frame.
     QSize winSize_, startSize_, prefSize_;
     QTimer *timer_;
+    QTimer *deactivateTimer_;
     QString pswrd_;
-    bool underE_; // Is FeatherNotes running under Enlightenment?
-    QSize EShift_; // The shift Enlightenment's panel creates (a bug?).
     QHash<QString, QString> customActions_;
     QHash<QAction*, QKeySequence> defaultShortcuts_;
     QStringList uncustomizedActions_, reservedShortcuts_;
