@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2016-2021 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2016-2024 <tsujan2000@gmail.com>
  *
  * FeatherNotes is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -15,29 +15,20 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <csignal> // signal.h in C
-#include <QStringList>
-#include <QApplication>
+#include <QTextStream>
+
+#include "singleton.h"
+#if not defined (Q_OS_WIN)
+#include "signalDaemon.h"
+#endif
+
 #include <QLibraryInfo>
 #include <QTranslator>
-#include <QTextStream>
-#include "fn.h"
-
-void handleQuitSignals (const std::vector<int>& quitSignals)
-{
-    auto handler = [](int sig) ->void {
-        Q_UNUSED (sig);
-        QCoreApplication::quit();
-    };
-
-    for (int sig : quitSignals )
-        signal (sig, handler); // handle these signals by quitting gracefully
-}
 
 int main(int argc, char *argv[])
 {
     const QString name = "FeatherNotes";
-    const QString version = "0.10.0";
+    const QString version = "1.3.0";
     const QString option = QString::fromUtf8 (argv[1]);
     if (option == "--help" || option == "-h")
     {
@@ -54,24 +45,13 @@ int main(int argc, char *argv[])
     else if (option == "--version" || option == "-v")
     {
         QTextStream out (stdout);
-        out << name << " " << version
-#if (QT_VERSION >= QT_VERSION_CHECK(5,15,0))
-            << Qt::endl;
-#else
-            << endl;
-#endif
+        out << name << " " << version << Qt::endl;
         return 0;
     }
 
-    QApplication app (argc, argv);
-    app.setApplicationName (name);
-    app.setApplicationVersion (version);
-
-#if not defined (Q_OS_WIN)
-    handleQuitSignals ({SIGQUIT, SIGINT, SIGTERM, SIGHUP}); // -> https://en.wikipedia.org/wiki/Unix_signal
-#endif
-
-    app.setAttribute (Qt::AA_UseHighDpiPixmaps, true);
+    FeatherNotes::FNSingleton singleton (argc, argv);
+    singleton.setApplicationName (name);
+    singleton.setApplicationVersion (version);
 
     QStringList langs (QLocale::system().uiLanguages());
     QString lang; // bcp47Name() doesn't work under vbox
@@ -79,37 +59,51 @@ int main(int argc, char *argv[])
         lang = langs.first().replace ('-', '_');
 
     QTranslator qtTranslator;
-    if (!qtTranslator.load ("qt_" + lang, QLibraryInfo::location (QLibraryInfo::TranslationsPath)))
-    { // shouldn't be needed
-        if (!langs.isEmpty())
-        {
-            lang = langs.first().split (QLatin1Char ('_')).first();
-            qtTranslator.load ("qt_" + lang, QLibraryInfo::location (QLibraryInfo::TranslationsPath));
-        }
+    if (qtTranslator.load ("qt_" + lang, QLibraryInfo::path (QLibraryInfo::TranslationsPath)))
+        singleton.installTranslator (&qtTranslator);
+    else if (!langs.isEmpty())
+    {
+        lang = langs.first().split (QLatin1Char ('_')).first();
+        if (qtTranslator.load ("qt_" + lang, QLibraryInfo::path (QLibraryInfo::TranslationsPath)))
+            singleton.installTranslator (&qtTranslator);
     }
-    app.installTranslator (&qtTranslator);
 
     QTranslator FPTranslator;
 #if defined (Q_OS_HAIKU)
-    FPTranslator.load ("feathernotes_" + lang, qApp->applicationDirPath() + "/translations");
+    if (FPTranslator.load ("feathernotes_" + lang, qApp->applicationDirPath() + "/translations"))
 #elif defined (Q_OS_WIN)
-    FPTranslator.load ("feathernotes_" + lang, qApp->applicationDirPath() + "\\..\\data\\translations\\translations");
+    if (FPTranslator.load ("feathernotes_" + lang, qApp->applicationDirPath() + "\\..\\data\\translations\\translations"))
 #else
-    FPTranslator.load ("feathernotes_" + lang, QStringLiteral (DATADIR) + "/feathernotes/translations");
+    if (FPTranslator.load ("feathernotes_" + lang, QStringLiteral (DATADIR) + "/feathernotes/translations"))
 #endif
-    app.installTranslator (&FPTranslator);
-
-    QStringList message;
-    if (argc > 1)
     {
-        message << QString::fromUtf8 (argv[1]);
-        if (argc > 2)
-            message << QString::fromUtf8 (argv[2]);
+        singleton.installTranslator (&FPTranslator);
     }
 
-    FeatherNotes::FN w (message);
+    QStringList info;
+    if (argc > 1)
+    {
+        info << QString::fromUtf8 (argv[1]);
+        if (argc > 2)
+            info << QString::fromUtf8 (argv[2]);
+    }
 
-    QObject::connect (&app, &QCoreApplication::aboutToQuit, &w, &FeatherNotes::FN::quitting);
+    if (!singleton.isPrimaryInstance())
+    {
+        singleton.sendInfo (info); // is sent to the primary instance
+        return 0;
+    }
 
-    return app.exec();
+#if not defined (Q_OS_WIN)
+    // Handle SIGQUIT, SIGINT, SIGTERM and SIGHUP (-> https://en.wikipedia.org/wiki/Unix_signal).
+    FeatherNotes::signalDaemon D;
+    D.watchUnixSignals();
+    QObject::connect (&D, &FeatherNotes::signalDaemon::sigQUIT,
+                      &singleton, &FeatherNotes::FNSingleton::quitSignalReceived);
+#endif
+
+    QObject::connect (&singleton, &QCoreApplication::aboutToQuit, &singleton, &FeatherNotes::FNSingleton::quitting);
+    singleton.openFile (info);
+
+    return singleton.exec();
 }
