@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2016-2021 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2016-2024 <tsujan2000@gmail.com>
  *
  * FeatherNotes is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -58,7 +58,7 @@ public:
         header()->setSectionResizeMode (QHeaderView::ResizeToContents);
     }
 
-    virtual void scrollTo (const QModelIndex &index, QAbstractItemView::ScrollHint hint = QAbstractItemView::EnsureVisible) {
+    void scrollTo (const QModelIndex &index, QAbstractItemView::ScrollHint hint = QAbstractItemView::EnsureVisible) override {
         QTreeView::scrollTo (index, hint);
         if (index.isValid())
         { // ensure that the item is visible horizontally too
@@ -79,33 +79,29 @@ public:
 
 signals:
     void FNDocDropped (const QString &path);
+    void searchRequest();
 
 protected:
     /* see Qt -> "qabstractitemview.cpp" */
-    virtual QItemSelectionModel::SelectionFlags selectionCommand (const QModelIndex &index, const QEvent *event = nullptr) const {
-        Qt::KeyboardModifiers keyModifiers = Qt::NoModifier;
-        if (event)
+    QItemSelectionModel::SelectionFlags selectionCommand (const QModelIndex &index, const QEvent *event = nullptr) const override {
+        Qt::KeyboardModifiers keyModifiers = event != nullptr && event->isInputEvent()
+                                                 ? (static_cast<const QInputEvent*>(event))->modifiers()
+                                                 : Qt::NoModifier;
+
+        if (selectionMode() == QAbstractItemView::SingleSelection)
         {
-            switch (event->type()) {
-                case QEvent::MouseButtonDblClick:
-                case QEvent::MouseButtonPress:
-                case QEvent::MouseButtonRelease:
-                case QEvent::MouseMove:
-                case QEvent::KeyPress:
-                case QEvent::KeyRelease:
-                    keyModifiers = (static_cast<const QInputEvent*>(event))->modifiers();
-                    break;
-                default:
-                    keyModifiers = QApplication::keyboardModifiers();
+            if (!index.isValid())
+                return QItemSelectionModel::NoUpdate;
+            if ((keyModifiers & Qt::ControlModifier)
+                && selectionModel()->isSelected (index))
+            {
+                return QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows;
             }
         }
-        if (selectionMode() == QAbstractItemView::SingleSelection && (keyModifiers & Qt::ControlModifier) && selectionModel()->isSelected (index))
-            return QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows;
-        else
-            return QTreeView::selectionCommand (index, event);
+        return QTreeView::selectionCommand (index, event);
     }
 
-    virtual void dragEnterEvent (QDragEnterEvent *event) {
+    void dragEnterEvent (QDragEnterEvent *event) override {
         if (event->mimeData()->hasUrls())
         {
             bool FNDocFound (false);
@@ -136,7 +132,7 @@ protected:
         QTreeView::dragEnterEvent (event);
     }
 
-    virtual void dropEvent (QDropEvent *event) {
+    void dropEvent (QDropEvent *event) override {
         if (event->mimeData()->hasUrls())
         {
             const auto urls = event->mimeData()->urls();
@@ -159,27 +155,43 @@ protected:
         QTreeView::dropEvent (event);
     }
 
-    virtual void mousePressEvent (QMouseEvent *event) {
+    void mousePressEvent (QMouseEvent *event) override {
+        QModelIndex index = indexAt (event->pos());
         /* get the global press position if it's inside an item to know
            whether there will be a real mouse movement at mouseMoveEvent() */
         if (event->buttons() == Qt::LeftButton
             && qApp->keyboardModifiers() == Qt::NoModifier)
         {
-            if (indexAt (event->pos()).isValid())
-                itemPressPoint_ = event->globalPos();
+            if (index.isValid())
+                itemPressPoint_ = event->globalPosition().toPoint();
             else itemPressPoint_ = QPoint();
         }
         else itemPressPoint_ = QPoint();
         QTreeView::mousePressEvent (event);
+
+        /* immediately scroll to the index if it's the current index
+           ("QAbstractItemView::mousePressEvent()" adds a delay) */
+        if (index.isValid() && index == currentIndex())
+            scrollTo (index);
     }
 
-    virtual void mouseMoveEvent (QMouseEvent *event) {
+    void mouseMoveEvent (QMouseEvent *event) override {
         /* prevent dragging if there is no real mouse movement */
-        if (event->buttons() == Qt::LeftButton && event->globalPos() == itemPressPoint_)
+        if (event->buttons() == Qt::LeftButton && (event->globalPosition().toPoint() - itemPressPoint_).manhattanLength() < QApplication::startDragDistance())
             return;
         QTreeView::mouseMoveEvent (event);
     }
 
+    void keyPressEvent (QKeyEvent *event) override {
+        if (event->key() == Qt::Key_Slash)
+        {
+            emit searchRequest();
+            event->accept();
+            return;
+        }
+        QTreeView::keyPressEvent (event);
+    }
+
 private:
     QPoint itemPressPoint_;
 };
